unit execunit;
(* ---------------------------------------------------------------------------

Purpose:Interface for encapsulation of WinAPI ShellEcexute-procedure.
  16.03.2002 / Miika Nurminen

Change: Execute procdure now throws EExecError exception.
  25.8.2002 / MN


Huom: Poluista (ks. gko, 5. luennon materiaali)

Sovelluksen kynnistyspolun saa selville seuraavilla tavoilla:
 * .\ alkuun tai
 * ParamStr(0) alkuun tai
 * Application.ExeName
 * getCurrentDir-funktiosta saattaa mys olla apua...
 * ParamStr:ssa ja ExeNamessa EXE-tiedoston nimi on poistettava
   ExtractFileDir-funktiolla

 GetCurrentDir on Delphin standardifunktio, palauttaa tmnhetkisen
 hakemiston.

 Listietoa ShellExecutesta: WindowsSDK: ShellExecute
 tai yleisemmin Shell library, ShellAPI
  4.6.2004 / MN
--------------------------------------------------------------------------- *)
interface

uses windows,ShellAPI,sysutils;

type
  EExecError = class(Exception)
  end;

procedure Exec(filename,dir,parameters:string;action:string='open'); overload;
procedure Exec(filename,dir:string); overload;
procedure Exec(filename:string); overload;

implementation

uses forms,errorlib;

// Kynnist tmnhetkisess hakemistossa (yleens?) sovelluksen
// kynnistyshakemisto olevassa tiedoston
procedure Exec(filename:string);
begin
  Exec(filename,GetCurrentDir+'\','');
end;

procedure Exec(filename,dir:string);
begin
  Exec(filename,dir,'');
end;


// Parameters: komentoriviparametrit.
// Action: open, print tai explore
// Heitt poikkeuksen ja avaa dialogin, jos tiedoston
// avaamisessa on ongelmia
procedure Exec(filename,dir,parameters,action:string);
var res:HINST;
begin
  res:=ShellExecute(Application.Handle,Pchar(action),PChar(filename),PChar(parameters),PChar(dir),SW_SHOW);

  if res>32 then Exit; // success
  case res of
    ERROR_FILE_NOT_FOUND,ERROR_PATH_NOT_FOUND,SE_ERR_DLLNOTFOUND(*,SE_ERR_FNF,SE_ERR_PNF*):
      ErrorDialog('Tiedostoa '+dir+filename+' ei lydy.','Tiedoston avaus',EExecError.Create(''));
    SE_ERR_ACCESSDENIED,SE_ERR_SHARE:
      ErrorDialog('Tiedostoon '+dir+filename+' ei saatu lukuoikeutta.','Tiedoston avaus',EExecError.Create(''));
    ERROR_BAD_FORMAT,SE_ERR_ASSOCINCOMPLETE,SE_ERR_NOASSOC:
      ShellExecute(Application.Handle,PChar(action),PChar('explorer'),PChar(dir+filename),nil,SW_SHOW);
  else
    ErrorDialog('Tuntematon virhe tiedostoa '+dir+filename+' luettaessa.','Tiedoston avaus',EExecError.Create(''));
  end;
end;

end.
