unit errorlib;
(* ----------------------------------------------------------------------------

Description:  General-purpose debug, testing & error handling routines

Miika Nurminen, jun 2001

To do:
 General logging/error framework class. Debug strings could be
 redirected to ODS or file.

---------------------------------------------------------------------------- *)
interface

uses sysutils;

type

{
  Exception is thrown when public method is called when object state is
  not appropriate.
}
EIllegalStateException = class(Exception)
end;

{
  Exception is thrown when public method is called
  with inappropriate arguments.
}
EIllegalArgumentException = class(Exception)
end;

procedure ODS(str:string);
procedure AssignError(o:TObject);
procedure FieldTypeError;
procedure TestNil(o:TObject;msg:string='');
procedure TestIntfNil(i:IUnknown;msg:string='');
procedure CheckBounds(o:TObject;value,overmax:integer;min:integer=0);
procedure ClassError(o:TObject;msg:string='');

procedure ErrorDialog(data,errlabel:string;e:Exception); // raises exception

resourcestring
  SUnknownFieldTypeError = 'Unknown Field Type!';

implementation

uses windows,forms;

// Raises exception with class information in msg
procedure ClassError(o:TObject;msg:string='');
begin
  raise Exception.Create(o.ClassName+': '+msg);
end;

procedure CheckBounds(o:TObject;value,overmax:integer;min:integer=0);
begin
    if (value<min) or (value>=overmax) then raise Exception.Create(o.ClassName+' out of bounds!');
end;

procedure FieldTypeError;
begin
  raise Exception.Create(SUnknownFieldTypeError);
end;

// Standard Windows Output Debug String
procedure ODS(str:string);
begin
    OutputDebugString(PChar(str));
end;

procedure AssignError(o:TObject);
begin
  raise Exception.Create(o.ClassName+' cannot be assigned!');
end;

procedure TestNil(o:TObject;msg:string);
begin
  if Length(msg)=0 then msg:='Object not assigned!';
  if not Assigned(o) then raise Exception.Create(msg);
end;

procedure TestIntfNil(i:IUnknown;msg:string=''); overload;
begin
  if Length(msg)=0 then msg:='Interface not assigned!';
  if i=nil then raise Exception.Create(msg);
end;

procedure ErrorDialog(data,errlabel:string;e:Exception);
begin
  Application.MessageBox(PChar(data),PChar(errlabel));
  if e.Message='' then e.Message:=data;
  raise e;
end;

end.
