/**************/
/* strlib.c   */
/****************************************************************************
    LIBRARY: STRLIB.C
    PURPOSE: "Malli DLL-kirjaston" tyosa, joka voidaan liitt sellaisenaan
             mys tavalliseksi kirjastoksi.
    Editor:  Vesa Lappalainen  10.11.1992
    USAGE:
      1) Listn omaan ohjelmaan
           #include "strlib.h"
      2) Kutsut aliohjelmiin:
           KysyJono(...)

=============================
Aliohjelmien tarkempi kuvaus:
=============================
  ------------------------------------------------------
  int WINAPI_LIB KysyJono(LPSTR kysymys, LPSTR oletus,
                          LPSTR jono, int max_pit)
  ------------------------------------------------------
    Kysytn merkkijono kyttjlt.  Kysymykseen tulee dialogi, jonka
    otsikkona on kysymys.  Dialogissa on kaksi nappulaa OK ja CANCEL.
    Mikli valitaan OK (=Return), palautetaan vastatusta jonosta niin
    monta merkki kuin tulokseen sopii.  Muuten (CANCEL = ESC) ei
    tulosta muuteta.

    Palauttaa:
      -1           = painettiin ESC tai CANCEL
      0           <= annetun merkkijonon pituus
    LPSTR kysymys  - dialogin otsikoksi tulostuva kysymys
    LPSTR oletus   - edit-kenttn oletuksena tulostuva arvo
    LPSTR jono     - tulosjono
    int max_pit    - tulosjonon suurin sallittu pituus (mys \0)

    Tyyppi WINAPI_LIB on joko WINAPI tai WINAPI _export
    sen mukaan, tuleeko tst .DLL vai ei.

  ------------------------------------------------------
  int WINAPI_LIB KysyJonoPaikka(int x,int y);
  ------------------------------------------------------
    Mr kysymysdialogin paikan.
    Jollei tt kutusta, niin sama paikka kuin ed. kerralla ji.

********************************************************************************
Tmn kirjaston kyttmiseksi tavallisena kirjastona pit projektissa olla
  Pohjelma.cpp
  ALI\owl\strlib.cpp (tai strlib.obj tai .lib, jossa tm on mukana)
  ALI\strlib.rc
  ???.def

Lisksi tarvitaan strlib.h

********************************************************************************
KIRJASTOISTA (*.lib ja *.dll):
==============================

Jos moduli halutaan list johonkin kirjastoon, pit tm moduli
ensin knt oikealla muistimallilla strlib.obj ja sitten
objekti listn kirjastoon vaikkapa TLIB -komennolla:

  TLIB oma.lib +-strlib.obj

eli otetaan vanha pois ja listn uusi tilalle (jos ei ole
ennestn, tulee tietysti varoitus ettei vanhaa voi poistaa).

Jos tst kirjastosta halutaan .dll, niin ks. dllmain.c

********************************************************************************

--------------------------------------------------------------------------------
Toteutuksesta:
==============

  Mys kutsunut ikkuna selvitetn GetFocus-kutsulla, jottei sit tarvitse
  tuoda parametrina.

  Jostain syyst dialogin CloseWindow(-1) kaataa ohjelman.  Tst syyst
  merkkijonon pituus lasketaan vasta kutsufunktiossa.

  Myskn dialogissa TWindow::Attr.X ei anna oikeaa tulosta.
  Ja WM_MOVE -viestin pt on client-nurkan koordinaatti => hankaluuksia
  dialogin saamiseksi oikealle paikalleen.

--------------------------------------------------------------------------------
Tehtvi:  1) Knn tst .dll versio ja kokeile toimintaa useammalla
              eri pohjelman esiintymll (eli sytt kesken kahdessa
              ohjelmassa yhtaikaa). Toimiiko?

*******************************************************************************/
#include <owl\owlpch.h>
#include "strlib.h"

#ifdef __DLL__
# include "dllmain.h"
#endif

#define NOPOS -9999
/******************************************************************************/
class TDlgPos { // Luokka, joka tallettaa ikkunan paikan
  TPoint p;
public:
  TDlgPos() : p(NOPOS,NOPOS) {}
  int Set(const TPoint &pt)  { p = pt; return 0;                               }
  int Move(TWindow &wnd)     { if ( p.x == NOPOS ) return 1;
                               wnd.SetWindowPos(NULL,p.x,p.y,0,0,
                                                SWP_NOSIZE | SWP_NOZORDER);
                               return 0;                                       }
  int Query(TWindow &wnd)    { TRect rc; wnd.GetWindowRect(rc);
                               p.x = rc.left; p.y = rc.top; return 0;          }
};

static TDlgPos DlgPos; // Staattinen dialogin edellisen kerran paikka


//------------------------------------------------------------------------------
class TAskStrDialog : public TDialog {
  const char *prompt;
  const char *def;
  char *str;
  int maxlen;
public:
  TAskStrDialog(TWindow *parent,const char *p,const char *d, char *s, int m) :
    TDialog(parent,"KYSYJONO"), prompt(p),def(d),str(s),maxlen(m)
                             {                                                 }
  virtual ~TAskStrDialog()   {                                                 }
  void SetupWindow()         { TDialog::SetupWindow();
                               SetWindowText(prompt);
                               SetDlgItemText(EDITJONO,def);
                               DlgPos.Move(*this);
                             }
  void BNOk()                { GetDlgItemText(EDITJONO,str,maxlen);
                               CloseWindow(IDOK);
                             }
//  void EvMove(TPoint &pt)    { DlgPos.Set(pt);                                 }
  void EvMove(TPoint &pt)    { DlgPos.Query(*this);                            }
//  void EvMove(TPoint &pt)    { DlgPos.Set(
//                                 TPoint(TWindow::Attr.X,TWindow::Attr.Y));     }
  DECLARE_RESPONSE_TABLE(TAskStrDialog);
};

DEFINE_RESPONSE_TABLE1(TAskStrDialog, TDialog)
  EV_WM_MOVE,
  EV_BN_CLICKED(IDOK,BNOk),
END_RESPONSE_TABLE;


/******************************************************************************/
int WINAPI_LIB KysyJono(LPSTR kysymys, LPSTR oletus,
                        LPSTR jono, int max_pit)
{
  TWindow wnd(GetFocus());
  int ret=TAskStrDialog(&wnd,kysymys,oletus,jono,max_pit).Execute();
  if ( ret != IDOK ) return -1;
  return strlen(jono);
}

/******************************************************************************/
int WINAPI_LIB KysyJonoPaikka(int x,int y)
{
  return DlgPos.Set(TPoint(x,y));
}