unit streamlib;
(* ---------------------------------------------------------------------------

Streamlib

purpose:     General-purpose stream functions
started:     9.7.2001
author:      Miika Nurminen

--------------------------------------------------------------------------- *)
interface

uses
    sysutils,classes,ADOdb,windows;




procedure ComponentToDFM(Component: TComponent; filename:string);
function DFMToComponent(filename: string): TComponent;
function ConvertBackSlash(s:string) : string;
procedure FreeAll(a:array of TObject);
function ReformatPath(s:string):string;

implementation

procedure ComponentToDFM(Component: TComponent; filename:string);
var
  BinStream:TMemoryStream;
  FStream: TFileStream;
begin
    if csDesigning in Component.ComponentState then exit;
  BinStream := TMemoryStream.Create;
  try
    FStream := TFileStream.Create(filename,fmCreate);
    try
      BinStream.WriteComponent(Component);
      BinStream.Seek(0, soFromBeginning);
      ObjectBinaryToText(BinStream, FStream);
    finally
      FStream.Free;
    end;
  finally
    BinStream.Free;
  end;
end;

function DFMToComponent(filename: string): TComponent;
var
  FStream:TFileStream;
  BinStream: TMemoryStream;
begin
    Result:=nil;
  try
    FStream := TFileStream.Create(filename,fmOpenRead);
    BinStream := TMemoryStream.Create;
    try
      ObjectTextToBinary(FStream, BinStream);
      BinStream.Seek(0, soFromBeginning);
      Result := BinStream.ReadComponent(nil);
    finally
        BinStream.Free;
        FStream.Free;
    end;
  except
  end;
end;


// Converts strings with '-chars into \' -format
function ConvertBackSlash(s:string) : string;
begin
    Result:=StringReplace(s,'''','\''',[rfReplaceAll]);
end;



// calls free for all members of a
procedure FreeAll(a:array of TObject);
var i:integer;
begin
    for i:=0 to High(a) do if Assigned(a[i]) then FreeAndNil(a[i]);
end;

{
  Converts forwardslashes to windows-styled backslashes
}
function ReformatPath(s:string):string;
begin
  Result:=StringReplace(s,'/','\',[rfReplaceAll]);
end;


function StripDriveLetter(s:string):string;
var t:string;
begin
    t:=ExtractFileDrive(s);
    if Length(t)>0 then
      s:=StringReplace(s,t+'\','',[]);
    Result:=s;
end;

end.
